﻿namespace CustomBehavior.Views.Behaviors
{
    using System;
    using System.Windows;
    using System.Windows.Controls;

    public class ReadFileBehavior
    {
        #region IsAttached 添付プロパティ定義
        public static readonly DependencyProperty IsAttachedProperty = DependencyProperty.RegisterAttached("IsAttached", typeof(bool), typeof(ReadFileBehavior), new FrameworkPropertyMetadata(false, OnIsAttachedChanged));
        public static bool GetIsAttached(DependencyObject target)
        {
            return (bool)target.GetValue(IsAttachedProperty);
        }
        public static void SetIsAttached(DependencyObject target, bool value)
        {
            target.SetValue(IsAttachedProperty, value);
        }
        #endregion IsAttached 添付プロパティ定義

        #region Callback 添付プロパティ定義
        public static readonly DependencyProperty CallbackProperty = DependencyProperty.RegisterAttached("Callback", typeof(Action<string>), typeof(ReadFileBehavior), new FrameworkPropertyMetadata(null));
        public static Action<string> GetCallback(DependencyObject target)
        {
            return (Action<string>)target.GetValue(CallbackProperty);
        }
        public static void SetCallback(DependencyObject target, Action<string> value)
        {
            target.SetValue(CallbackProperty, value);
        }
        #endregion Callback 添付プロパティ定義

        /// <summary>
        /// IsAttached 添付プロパティ値変更イベントハンドラ
        /// </summary>
        /// <param name="d">イベント発生元</param>
        /// <param name="e">イベント引数</param>
        private static void OnIsAttachedChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var button = d as Button;
            if (button != null)
            {
                if (GetIsAttached(button))
                {
                    // IsAttached 添付プロパティが true の場合はイベントハンドラを登録する
                    button.Click += OnClick;
                }
                else
                {
                    // IsAttached 添付プロパティが false の場合はイベントハンドラを登録解除する
                    button.Click -= OnClick;
                }
            }
        }

        /// <summary>
        /// Click イベントハンドラ
        /// </summary>
        /// <param name="sender">イベント発生元</param>
        /// <param name="e">イベント引数</param>
        private static void OnClick(object sender, RoutedEventArgs e)
        {
            var button = sender as Button;
            if (button != null)
            {
                var dlg = new Microsoft.Win32.OpenFileDialog();
                var result = dlg.ShowDialog();
                if ((result != null) && (result == true))
                {
                    // ファイルのフルパスを取得
                    var filename = dlg.FileName;

                    // ファイルを開く処理
                    var callback = GetCallback(button);
                    if (callback != null)
                        callback(filename);
                }
            }
        }
    }
}
